;;; -*- coding: utf-8-unix -*-
;;;
;;;Part of: Vicare/OpenSSL
;;;Contents: static platform inspection
;;;Date: Sat Mar  9, 2013
;;;
;;;Abstract
;;;
;;;
;;;
;;;Copyright (C) 2013 Marco Maggi <marco.maggi-ipsu@poste.it>
;;;
;;;This program is free software:  you can redistribute it and/or modify
;;;it under the terms of the  GNU General Public License as published by
;;;the Free Software Foundation, either version 3 of the License, or (at
;;;your option) any later version.
;;;
;;;This program is  distributed in the hope that it  will be useful, but
;;;WITHOUT  ANY   WARRANTY;  without   even  the  implied   warranty  of
;;;MERCHANTABILITY or  FITNESS FOR  A PARTICULAR  PURPOSE.  See  the GNU
;;;General Public License for more details.
;;;
;;;You should  have received a  copy of  the GNU General  Public License
;;;along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;;


#!r6rs
(library (vicare crypto openssl features)
  (export
    ;; Library initialisation features
    HAVE_SSL_LIBRARY_INIT
    HAVE_OPENSSL_ADD_ALL_ALGORITHMS_NOCONF
    HAVE_OPENSSL_ADD_ALL_ALGORITHMS_CONF
    HAVE_DECL_OPENSSL_ADD_ALL_ALGORITHMS
    HAVE_OPENSSL_ADD_ALL_CIPHERS
    HAVE_OPENSSL_ADD_ALL_DIGESTS
    HAVE_DECL_SSLEAY_ADD_ALL_ALGORITHMS
    HAVE_DECL_SSLEAY_ADD_ALL_CIPHERS
    HAVE_DECL_SSLEAY_ADD_ALL_DIGESTS
    
    HAVE_MD4_INIT
    HAVE_MD4_UPDATE
    HAVE_MD4_FINAL
    HAVE_MD4
    
    HAVE_MD5_INIT
    HAVE_MD5_UPDATE
    HAVE_MD5_FINAL
    HAVE_MD5
    
    ;; MDC2 features
    HAVE_MDC2_INIT
    HAVE_MDC2_UPDATE
    HAVE_MDC2_FINAL
    HAVE_MDC2
    
    ;; SHA 1 features
    HAVE_SHA1_INIT
    HAVE_SHA1_UPDATE
    HAVE_SHA1_FINAL
    HAVE_SHA1
    
    ;; SHA 224 features
    HAVE_SHA224_INIT
    HAVE_SHA224_UPDATE
    HAVE_SHA224_FINAL
    HAVE_SHA224
    
    ;; SHA 256 features
    HAVE_SHA256_INIT
    HAVE_SHA256_UPDATE
    HAVE_SHA256_FINAL
    HAVE_SHA256
    
    ;; SHA 384 features
    HAVE_SHA384_INIT
    HAVE_SHA384_UPDATE
    HAVE_SHA384_FINAL
    HAVE_SHA384
    
    ;; SHA 512 features
    HAVE_SHA512_INIT
    HAVE_SHA512_UPDATE
    HAVE_SHA512_FINAL
    HAVE_SHA512
    
    HAVE_RIPEMD160_INIT
    HAVE_RIPEMD160_UPDATE
    HAVE_RIPEMD160_FINAL
    HAVE_RIPEMD160
    
    HAVE_WHIRLPOOL_INIT
    HAVE_WHIRLPOOL_UPDATE
    HAVE_WHIRLPOOL_FINAL
    HAVE_WHIRLPOOL
    
    ;; HMAC features
    HAVE_HMAC
    HAVE_HMAC_CTX_INIT
    HAVE_HMAC_CTX_CLEANUP
    HAVE_HMAC_INIT
    HAVE_HMAC_INIT_EX
    HAVE_HMAC_UPDATE
    HAVE_HMAC_FINAL
    HAVE_HMAC_CTX_COPY
    HAVE_HMAC_CTX_SET_FLAGS
    
    ;; EVP hash functions features
    HAVE_EVP_MD_TYPE
    HAVE_EVP_MD_NID
    HAVE_EVP_MD_NAME
    HAVE_EVP_MD_PKEY_TYPE
    HAVE_EVP_MD_SIZE
    HAVE_EVP_MD_BLOCK_SIZE
    HAVE_EVP_MD_FLAGS
    HAVE_EVP_MD_CTX_MD
    HAVE_EVP_MD_CTX_SIZE
    HAVE_EVP_MD_CTX_BLOCK_SIZE
    HAVE_EVP_MD_CTX_TYPE
    HAVE_EVP_MD_CTX_INIT
    HAVE_EVP_MD_CTX_CLEANUP
    HAVE_EVP_MD_CTX_CREATE
    HAVE_EVP_MD_CTX_DESTROY
    HAVE_EVP_MD_CTX_COPY_EX
    HAVE_EVP_MD_CTX_SET_FLAGS
    HAVE_EVP_MD_CTX_CLEAR_FLAGS
    HAVE_EVP_MD_CTX_TEST_FLAGS
    HAVE_EVP_DIGESTINIT_EX
    HAVE_EVP_DIGESTUPDATE
    HAVE_EVP_DIGESTFINAL_EX
    HAVE_EVP_DIGEST
    HAVE_EVP_MD_CTX_COPY
    HAVE_EVP_DIGESTINIT
    HAVE_EVP_DIGESTFINAL
    HAVE_EVP_MD_NULL
    HAVE_EVP_MD2
    HAVE_EVP_MD4
    HAVE_EVP_MD5
    HAVE_EVP_SHA
    HAVE_EVP_SHA1
    HAVE_EVP_DSS
    HAVE_EVP_DSS1
    HAVE_EVP_ECDSA
    HAVE_EVP_SHA224
    HAVE_EVP_SHA256
    HAVE_EVP_SHA384
    HAVE_EVP_SHA512
    HAVE_EVP_MDC2
    HAVE_EVP_RIPEMD160
    HAVE_EVP_WHIRLPOOL
    HAVE_EVP_GET_DIGESTBYNAME
    
    ;; EVP cipher algorithms features
    HAVE_EVP_ENC_NULL
    HAVE_EVP_DES_ECB
    HAVE_EVP_DES_EDE
    HAVE_EVP_DES_EDE3
    HAVE_EVP_DES_EDE_ECB
    HAVE_EVP_DES_EDE3_ECB
    HAVE_EVP_DES_CFB64
    HAVE_DECL_EVP_DES_CFB
    HAVE_EVP_DES_EDE3_CFB64
    HAVE_DECL_EVP_DES_EDE3_CFB
    HAVE_EVP_DES_EDE3_CFB1
    HAVE_EVP_DES_EDE3_CFB8
    HAVE_EVP_DES_OFB
    HAVE_EVP_DES_EDE_OFB
    HAVE_EVP_DES_EDE3_OFB
    HAVE_EVP_DES_CBC
    HAVE_EVP_DES_EDE_CBC
    HAVE_EVP_DES_EDE3_CBC
    HAVE_EVP_DESX_CBC
    HAVE_EVP_RC4
    HAVE_EVP_RC4_40
    HAVE_EVP_RC4_HMAC_MD5
    HAVE_EVP_IDEA_ECB
    HAVE_EVP_IDEA_CFB64
    HAVE_DECL_EVP_IDEA_CFB
    HAVE_EVP_IDEA_OFB
    HAVE_EVP_IDEA_CBC
    HAVE_EVP_RC2_ECB
    HAVE_EVP_RC2_CBC
    HAVE_EVP_RC2_40_CBC
    HAVE_EVP_RC2_64_CBC
    HAVE_EVP_RC2_CFB64
    HAVE_DECL_EVP_RC2_CFB
    HAVE_EVP_RC2_OFB
    HAVE_EVP_BF_ECB
    HAVE_EVP_BF_CBC
    HAVE_EVP_BF_CFB64
    HAVE_DECL_EVP_BF_CFB
    HAVE_EVP_BF_OFB
    HAVE_EVP_CAST5_ECB
    HAVE_EVP_CAST5_CBC
    HAVE_EVP_CAST5_CFB64
    HAVE_DECL_EVP_CAST5_CFB
    HAVE_EVP_CAST5_OFB
    HAVE_EVP_RC5_32_12_16_CBC
    HAVE_EVP_RC5_32_12_16_ECB
    HAVE_EVP_RC5_32_12_16_CFB64
    HAVE_DECL_EVP_RC5_32_12_16_CFB
    HAVE_EVP_RC5_32_12_16_OFB
    HAVE_EVP_AES_128_ECB
    HAVE_EVP_AES_128_CBC
    HAVE_EVP_AES_128_CFB1
    HAVE_EVP_AES_128_CFB8
    HAVE_EVP_AES_128_CFB128
    HAVE_DECL_EVP_AES_128_CFB
    HAVE_EVP_AES_128_OFB
    HAVE_EVP_AES_128_CTR
    HAVE_EVP_AES_128_CCM
    HAVE_EVP_AES_128_GCM
    HAVE_EVP_AES_128_XTS
    HAVE_EVP_AES_192_ECB
    HAVE_EVP_AES_192_CBC
    HAVE_EVP_AES_192_CFB1
    HAVE_EVP_AES_192_CFB8
    HAVE_EVP_AES_192_CFB128
    HAVE_DECL_EVP_AES_192_CFB
    HAVE_EVP_AES_192_OFB
    HAVE_EVP_AES_192_CTR
    HAVE_EVP_AES_192_CCM
    HAVE_EVP_AES_192_GCM
    HAVE_EVP_AES_256_ECB
    HAVE_EVP_AES_256_CBC
    HAVE_EVP_AES_256_CFB1
    HAVE_EVP_AES_256_CFB8
    HAVE_EVP_AES_256_CFB128
    HAVE_DECL_EVP_AES_256_CFB
    HAVE_EVP_AES_256_OFB
    HAVE_EVP_AES_256_CTR
    HAVE_EVP_AES_256_CCM
    HAVE_EVP_AES_256_GCM
    HAVE_EVP_AES_256_XTS
    HAVE_EVP_AES_128_CBC_HMAC_SHA1
    HAVE_EVP_AES_256_CBC_HMAC_SHA1
    HAVE_EVP_CAMELLIA_128_ECB
    HAVE_EVP_CAMELLIA_128_CBC
    HAVE_EVP_CAMELLIA_128_CFB1
    HAVE_EVP_CAMELLIA_128_CFB8
    HAVE_EVP_CAMELLIA_128_CFB128
    HAVE_DECL_EVP_CAMELLIA_128_CFB
    HAVE_EVP_CAMELLIA_128_OFB
    HAVE_EVP_CAMELLIA_192_ECB
    HAVE_EVP_CAMELLIA_192_CBC
    HAVE_EVP_CAMELLIA_192_CFB1
    HAVE_EVP_CAMELLIA_192_CFB8
    HAVE_EVP_CAMELLIA_192_CFB128
    HAVE_DECL_EVP_CAMELLIA_192_CFB
    HAVE_EVP_CAMELLIA_192_OFB
    HAVE_EVP_CAMELLIA_256_ECB
    HAVE_EVP_CAMELLIA_256_CBC
    HAVE_EVP_CAMELLIA_256_CFB1
    HAVE_EVP_CAMELLIA_256_CFB8
    HAVE_EVP_CAMELLIA_256_CFB128
    HAVE_DECL_EVP_CAMELLIA_256_CFB
    HAVE_EVP_CAMELLIA_256_OFB
    HAVE_EVP_SEED_ECB
    HAVE_EVP_SEED_CBC
    HAVE_EVP_SEED_CFB128
    HAVE_DECL_EVP_SEED_CFB
    HAVE_EVP_SEED_OFB
    HAVE_EVP_CIPHER_TYPE
    HAVE_EVP_GET_CIPHERBYNAME
    HAVE_DECL_EVP_GET_CIPHERBYNID
    HAVE_DECL_EVP_GET_CIPHERBYOBJ
    HAVE_EVP_CIPHER_NID
    HAVE_DECL_EVP_CIPHER_NAME
    HAVE_EVP_CIPHER_BLOCK_SIZE
    HAVE_EVP_CIPHER_KEY_LENGTH
    HAVE_EVP_CIPHER_IV_LENGTH
    HAVE_EVP_CIPHER_FLAGS
    HAVE_DECL_EVP_CIPHER_MODE
    HAVE_EVP_CIPHER_CTX_INIT
    HAVE_EVP_CIPHER_CTX_CLEANUP
    HAVE_EVP_CIPHER_CTX_NEW
    HAVE_EVP_CIPHER_CTX_FREE
    HAVE_EVP_ENCRYPTINIT_EX
    HAVE_EVP_ENCRYPTFINAL_EX
    HAVE_EVP_ENCRYPTUPDATE
    HAVE_EVP_DECRYPTINIT_EX
    HAVE_EVP_DECRYPTUPDATE
    HAVE_EVP_DECRYPTFINAL_EX
    HAVE_EVP_CIPHERINIT_EX
    HAVE_EVP_CIPHERUPDATE
    HAVE_EVP_CIPHERFINAL_EX
    HAVE_EVP_CIPHER_CTX_SET_KEY_LENGTH
    HAVE_EVP_CIPHER_CTX_SET_PADDING
    HAVE_EVP_CIPHER_CTX_CTRL
    HAVE_EVP_CIPHER_CTX_CIPHER
    HAVE_EVP_CIPHER_CTX_NID
    HAVE_EVP_CIPHER_CTX_BLOCK_SIZE
    HAVE_EVP_CIPHER_CTX_KEY_LENGTH
    HAVE_EVP_CIPHER_CTX_IV_LENGTH
    HAVE_EVP_CIPHER_CTX_COPY
    HAVE_EVP_CIPHER_CTX_GET_APP_DATA
    HAVE_EVP_CIPHER_CTX_SET_APP_DATA
    HAVE_DECL_EVP_CIPHER_CTX_TYPE
    HAVE_EVP_CIPHER_CTX_FLAGS
    HAVE_DECL_EVP_CIPHER_CTX_MODE
    HAVE_EVP_CIPHER_CTX_RAND_KEY
    HAVE_EVP_CIPHER_PARAM_TO_ASN1
    HAVE_EVP_CIPHER_ASN1_TO_PARAM
    HAVE_EVP_CIPHER_CTX_SET_FLAGS
    HAVE_EVP_CIPHER_CTX_CLEAR_FLAGS
    HAVE_EVP_CIPHER_CTX_TEST_FLAGS
    HAVE_EVP_CIPHER
    
    ;; AES features
    HAVE_AES_OPTIONS
    HAVE_AES_SET_ENCRYPT_KEY
    HAVE_AES_SET_DECRYPT_KEY
    HAVE_AES_ENCRYPT
    HAVE_AES_DECRYPT
    HAVE_AES_ECB_ENCRYPT
    HAVE_AES_CBC_ENCRYPT
    HAVE_AES_CFB128_ENCRYPT
    HAVE_AES_CFB1_ENCRYPT
    HAVE_AES_CFB8_ENCRYPT
    HAVE_AES_OFB128_ENCRYPT
    HAVE_AES_CTR128_ENCRYPT
    HAVE_AES_IGE_ENCRYPT
    HAVE_AES_BI_IGE_ENCRYPT
    HAVE_AES_WRAP_KEY
    HAVE_AES_UNWRAP_KEY
    
    ;; buffer features
    HAVE_BUF_MEM_NEW
    HAVE_BUF_MEM_FREE
    HAVE_BUF_MEM_GROW
    HAVE_BUF_MEM_GROW_CLEAN
    HAVE_BUF_STRDUP
    HAVE_BUF_STRNDUP
    HAVE_BUF_MEMDUP
    HAVE_BUF_REVERSE
    HAVE_BUF_STRLCPY
    HAVE_BUF_STRLCAT
    HAVE_ERR_LOAD_BUF_STRINGS
    
    )
  (import (rnrs))

;;;; helpers

(define-syntax define-inline-constant
  (syntax-rules ()
    ((_ ?name ?value)
     (define-syntax ?name (identifier-syntax ?value)))))


;;;; code

(define-inline-constant HAVE_SSL_LIBRARY_INIT #t)
(define-inline-constant HAVE_OPENSSL_ADD_ALL_ALGORITHMS_NOCONF #t)
(define-inline-constant HAVE_OPENSSL_ADD_ALL_ALGORITHMS_CONF #t)
(define-inline-constant HAVE_DECL_OPENSSL_ADD_ALL_ALGORITHMS #t)
(define-inline-constant HAVE_OPENSSL_ADD_ALL_CIPHERS #t)
(define-inline-constant HAVE_OPENSSL_ADD_ALL_DIGESTS #t)
(define-inline-constant HAVE_DECL_SSLEAY_ADD_ALL_ALGORITHMS #t)
(define-inline-constant HAVE_DECL_SSLEAY_ADD_ALL_CIPHERS #t)
(define-inline-constant HAVE_DECL_SSLEAY_ADD_ALL_DIGESTS #t)
(define-inline-constant HAVE_MD4_INIT #t)
(define-inline-constant HAVE_MD4_UPDATE #t)
(define-inline-constant HAVE_MD4_FINAL #t)
(define-inline-constant HAVE_MD4 #t)
(define-inline-constant HAVE_MD5_INIT #t)
(define-inline-constant HAVE_MD5_UPDATE #t)
(define-inline-constant HAVE_MD5_FINAL #t)
(define-inline-constant HAVE_MD5 #t)
(define-inline-constant HAVE_MDC2_INIT #f)
(define-inline-constant HAVE_MDC2_UPDATE #f)
(define-inline-constant HAVE_MDC2_FINAL #f)
(define-inline-constant HAVE_MDC2 #f)
(define-inline-constant HAVE_SHA1_INIT #t)
(define-inline-constant HAVE_SHA1_UPDATE #t)
(define-inline-constant HAVE_SHA1_FINAL #t)
(define-inline-constant HAVE_SHA1 #t)
(define-inline-constant HAVE_SHA224_INIT #t)
(define-inline-constant HAVE_SHA224_UPDATE #t)
(define-inline-constant HAVE_SHA224_FINAL #t)
(define-inline-constant HAVE_SHA224 #t)
(define-inline-constant HAVE_SHA256_INIT #t)
(define-inline-constant HAVE_SHA256_UPDATE #t)
(define-inline-constant HAVE_SHA256_FINAL #t)
(define-inline-constant HAVE_SHA256 #t)
(define-inline-constant HAVE_SHA384_INIT #t)
(define-inline-constant HAVE_SHA384_UPDATE #t)
(define-inline-constant HAVE_SHA384_FINAL #t)
(define-inline-constant HAVE_SHA384 #t)
(define-inline-constant HAVE_SHA512_INIT #t)
(define-inline-constant HAVE_SHA512_UPDATE #t)
(define-inline-constant HAVE_SHA512_FINAL #t)
(define-inline-constant HAVE_SHA512 #t)
(define-inline-constant HAVE_RIPEMD160_INIT #t)
(define-inline-constant HAVE_RIPEMD160_UPDATE #t)
(define-inline-constant HAVE_RIPEMD160_FINAL #t)
(define-inline-constant HAVE_RIPEMD160 #t)
(define-inline-constant HAVE_WHIRLPOOL_INIT #t)
(define-inline-constant HAVE_WHIRLPOOL_UPDATE #t)
(define-inline-constant HAVE_WHIRLPOOL_FINAL #t)
(define-inline-constant HAVE_WHIRLPOOL #t)
(define-inline-constant HAVE_HMAC #t)
(define-inline-constant HAVE_HMAC_CTX_INIT #t)
(define-inline-constant HAVE_HMAC_CTX_CLEANUP #t)
(define-inline-constant HAVE_HMAC_INIT #t)
(define-inline-constant HAVE_HMAC_INIT_EX #t)
(define-inline-constant HAVE_HMAC_UPDATE #t)
(define-inline-constant HAVE_HMAC_FINAL #t)
(define-inline-constant HAVE_HMAC_CTX_COPY #t)
(define-inline-constant HAVE_HMAC_CTX_SET_FLAGS #t)
(define-inline-constant HAVE_AES_OPTIONS #t)
(define-inline-constant HAVE_AES_SET_ENCRYPT_KEY #t)
(define-inline-constant HAVE_AES_SET_DECRYPT_KEY #t)
(define-inline-constant HAVE_AES_ENCRYPT #t)
(define-inline-constant HAVE_AES_DECRYPT #t)
(define-inline-constant HAVE_AES_ECB_ENCRYPT #t)
(define-inline-constant HAVE_AES_CBC_ENCRYPT #t)
(define-inline-constant HAVE_AES_CFB128_ENCRYPT #t)
(define-inline-constant HAVE_AES_CFB1_ENCRYPT #t)
(define-inline-constant HAVE_AES_CFB8_ENCRYPT #t)
(define-inline-constant HAVE_AES_OFB128_ENCRYPT #t)
(define-inline-constant HAVE_AES_CTR128_ENCRYPT #t)
(define-inline-constant HAVE_AES_IGE_ENCRYPT #t)
(define-inline-constant HAVE_AES_BI_IGE_ENCRYPT #t)
(define-inline-constant HAVE_AES_WRAP_KEY #t)
(define-inline-constant HAVE_AES_UNWRAP_KEY #t)
(define-inline-constant HAVE_EVP_MD_TYPE #t)
(define-inline-constant HAVE_EVP_MD_NID #f)
(define-inline-constant HAVE_EVP_MD_NAME #f)
(define-inline-constant HAVE_EVP_MD_PKEY_TYPE #t)
(define-inline-constant HAVE_EVP_MD_SIZE #t)
(define-inline-constant HAVE_EVP_MD_BLOCK_SIZE #t)
(define-inline-constant HAVE_EVP_MD_FLAGS #t)
(define-inline-constant HAVE_EVP_MD_CTX_MD #t)
(define-inline-constant HAVE_EVP_MD_CTX_SIZE #f)
(define-inline-constant HAVE_EVP_MD_CTX_BLOCK_SIZE #f)
(define-inline-constant HAVE_EVP_MD_CTX_TYPE #f)
(define-inline-constant HAVE_EVP_MD_CTX_INIT #t)
(define-inline-constant HAVE_EVP_MD_CTX_CLEANUP #t)
(define-inline-constant HAVE_EVP_MD_CTX_CREATE #t)
(define-inline-constant HAVE_EVP_MD_CTX_DESTROY #t)
(define-inline-constant HAVE_EVP_MD_CTX_COPY_EX #t)
(define-inline-constant HAVE_EVP_MD_CTX_SET_FLAGS #t)
(define-inline-constant HAVE_EVP_MD_CTX_CLEAR_FLAGS #t)
(define-inline-constant HAVE_EVP_MD_CTX_TEST_FLAGS #t)
(define-inline-constant HAVE_EVP_DIGESTINIT_EX #t)
(define-inline-constant HAVE_EVP_DIGESTUPDATE #t)
(define-inline-constant HAVE_EVP_DIGESTFINAL_EX #t)
(define-inline-constant HAVE_EVP_DIGEST #t)
(define-inline-constant HAVE_EVP_MD_CTX_COPY #t)
(define-inline-constant HAVE_EVP_DIGESTINIT #t)
(define-inline-constant HAVE_EVP_DIGESTFINAL #t)
(define-inline-constant HAVE_EVP_MD_NULL #t)
(define-inline-constant HAVE_EVP_MD2 #f)
(define-inline-constant HAVE_EVP_MD4 #t)
(define-inline-constant HAVE_EVP_MD5 #t)
(define-inline-constant HAVE_EVP_SHA #t)
(define-inline-constant HAVE_EVP_SHA1 #t)
(define-inline-constant HAVE_EVP_DSS #t)
(define-inline-constant HAVE_EVP_DSS1 #t)
(define-inline-constant HAVE_EVP_ECDSA #t)
(define-inline-constant HAVE_EVP_SHA224 #t)
(define-inline-constant HAVE_EVP_SHA256 #t)
(define-inline-constant HAVE_EVP_SHA384 #t)
(define-inline-constant HAVE_EVP_SHA512 #t)
(define-inline-constant HAVE_EVP_MDC2 #f)
(define-inline-constant HAVE_EVP_RIPEMD160 #t)
(define-inline-constant HAVE_EVP_WHIRLPOOL #t)
(define-inline-constant HAVE_EVP_GET_DIGESTBYNAME #t)
(define-inline-constant HAVE_EVP_ENC_NULL #t)
(define-inline-constant HAVE_EVP_DES_ECB #t)
(define-inline-constant HAVE_EVP_DES_EDE #t)
(define-inline-constant HAVE_EVP_DES_EDE3 #t)
(define-inline-constant HAVE_EVP_DES_EDE_ECB #t)
(define-inline-constant HAVE_EVP_DES_EDE3_ECB #t)
(define-inline-constant HAVE_EVP_DES_CFB64 #t)
(define-inline-constant HAVE_DECL_EVP_DES_CFB #t)
(define-inline-constant HAVE_EVP_DES_EDE3_CFB64 #t)
(define-inline-constant HAVE_DECL_EVP_DES_EDE3_CFB #t)
(define-inline-constant HAVE_EVP_DES_EDE3_CFB1 #t)
(define-inline-constant HAVE_EVP_DES_EDE3_CFB8 #t)
(define-inline-constant HAVE_EVP_DES_OFB #t)
(define-inline-constant HAVE_EVP_DES_EDE_OFB #t)
(define-inline-constant HAVE_EVP_DES_EDE3_OFB #t)
(define-inline-constant HAVE_EVP_DES_CBC #t)
(define-inline-constant HAVE_EVP_DES_EDE_CBC #t)
(define-inline-constant HAVE_EVP_DES_EDE3_CBC #t)
(define-inline-constant HAVE_EVP_DESX_CBC #t)
(define-inline-constant HAVE_EVP_RC4 #t)
(define-inline-constant HAVE_EVP_RC4_40 #t)
(define-inline-constant HAVE_EVP_RC4_HMAC_MD5 #t)
(define-inline-constant HAVE_EVP_IDEA_ECB #f)
(define-inline-constant HAVE_EVP_IDEA_CFB64 #f)
(define-inline-constant HAVE_DECL_EVP_IDEA_CFB #f)
(define-inline-constant HAVE_EVP_IDEA_OFB #f)
(define-inline-constant HAVE_EVP_IDEA_CBC #f)
(define-inline-constant HAVE_EVP_RC2_ECB #t)
(define-inline-constant HAVE_EVP_RC2_CBC #t)
(define-inline-constant HAVE_EVP_RC2_40_CBC #t)
(define-inline-constant HAVE_EVP_RC2_64_CBC #t)
(define-inline-constant HAVE_EVP_RC2_CFB64 #t)
(define-inline-constant HAVE_DECL_EVP_RC2_CFB #t)
(define-inline-constant HAVE_EVP_RC2_OFB #t)
(define-inline-constant HAVE_EVP_BF_ECB #t)
(define-inline-constant HAVE_EVP_BF_CBC #t)
(define-inline-constant HAVE_EVP_BF_CFB64 #t)
(define-inline-constant HAVE_DECL_EVP_BF_CFB #t)
(define-inline-constant HAVE_EVP_BF_OFB #t)
(define-inline-constant HAVE_EVP_CAST5_ECB #t)
(define-inline-constant HAVE_EVP_CAST5_CBC #t)
(define-inline-constant HAVE_EVP_CAST5_CFB64 #t)
(define-inline-constant HAVE_DECL_EVP_CAST5_CFB #t)
(define-inline-constant HAVE_EVP_CAST5_OFB #t)
(define-inline-constant HAVE_EVP_RC5_32_12_16_CBC #f)
(define-inline-constant HAVE_EVP_RC5_32_12_16_ECB #f)
(define-inline-constant HAVE_EVP_RC5_32_12_16_CFB64 #f)
(define-inline-constant HAVE_DECL_EVP_RC5_32_12_16_CFB #f)
(define-inline-constant HAVE_EVP_RC5_32_12_16_OFB #f)
(define-inline-constant HAVE_EVP_AES_128_ECB #t)
(define-inline-constant HAVE_EVP_AES_128_CBC #t)
(define-inline-constant HAVE_EVP_AES_128_CFB1 #t)
(define-inline-constant HAVE_EVP_AES_128_CFB8 #t)
(define-inline-constant HAVE_EVP_AES_128_CFB128 #t)
(define-inline-constant HAVE_DECL_EVP_AES_128_CFB #t)
(define-inline-constant HAVE_EVP_AES_128_OFB #t)
(define-inline-constant HAVE_EVP_AES_128_CTR #t)
(define-inline-constant HAVE_EVP_AES_128_CCM #t)
(define-inline-constant HAVE_EVP_AES_128_GCM #t)
(define-inline-constant HAVE_EVP_AES_128_XTS #t)
(define-inline-constant HAVE_EVP_AES_192_ECB #t)
(define-inline-constant HAVE_EVP_AES_192_CBC #t)
(define-inline-constant HAVE_EVP_AES_192_CFB1 #t)
(define-inline-constant HAVE_EVP_AES_192_CFB8 #t)
(define-inline-constant HAVE_EVP_AES_192_CFB128 #t)
(define-inline-constant HAVE_DECL_EVP_AES_192_CFB #t)
(define-inline-constant HAVE_EVP_AES_192_OFB #t)
(define-inline-constant HAVE_EVP_AES_192_CTR #t)
(define-inline-constant HAVE_EVP_AES_192_CCM #t)
(define-inline-constant HAVE_EVP_AES_192_GCM #t)
(define-inline-constant HAVE_EVP_AES_256_ECB #t)
(define-inline-constant HAVE_EVP_AES_256_CBC #t)
(define-inline-constant HAVE_EVP_AES_256_CFB1 #t)
(define-inline-constant HAVE_EVP_AES_256_CFB8 #t)
(define-inline-constant HAVE_EVP_AES_256_CFB128 #t)
(define-inline-constant HAVE_DECL_EVP_AES_256_CFB #t)
(define-inline-constant HAVE_EVP_AES_256_OFB #t)
(define-inline-constant HAVE_EVP_AES_256_CTR #t)
(define-inline-constant HAVE_EVP_AES_256_CCM #t)
(define-inline-constant HAVE_EVP_AES_256_GCM #t)
(define-inline-constant HAVE_EVP_AES_256_XTS #t)
(define-inline-constant HAVE_EVP_AES_128_CBC_HMAC_SHA1 #t)
(define-inline-constant HAVE_EVP_AES_256_CBC_HMAC_SHA1 #t)
(define-inline-constant HAVE_EVP_CAMELLIA_128_ECB #t)
(define-inline-constant HAVE_EVP_CAMELLIA_128_CBC #t)
(define-inline-constant HAVE_EVP_CAMELLIA_128_CFB1 #t)
(define-inline-constant HAVE_EVP_CAMELLIA_128_CFB8 #t)
(define-inline-constant HAVE_EVP_CAMELLIA_128_CFB128 #t)
(define-inline-constant HAVE_DECL_EVP_CAMELLIA_128_CFB #t)
(define-inline-constant HAVE_EVP_CAMELLIA_128_OFB #t)
(define-inline-constant HAVE_EVP_CAMELLIA_192_ECB #t)
(define-inline-constant HAVE_EVP_CAMELLIA_192_CBC #t)
(define-inline-constant HAVE_EVP_CAMELLIA_192_CFB1 #t)
(define-inline-constant HAVE_EVP_CAMELLIA_192_CFB8 #t)
(define-inline-constant HAVE_EVP_CAMELLIA_192_CFB128 #t)
(define-inline-constant HAVE_DECL_EVP_CAMELLIA_192_CFB #t)
(define-inline-constant HAVE_EVP_CAMELLIA_192_OFB #t)
(define-inline-constant HAVE_EVP_CAMELLIA_256_ECB #t)
(define-inline-constant HAVE_EVP_CAMELLIA_256_CBC #t)
(define-inline-constant HAVE_EVP_CAMELLIA_256_CFB1 #t)
(define-inline-constant HAVE_EVP_CAMELLIA_256_CFB8 #t)
(define-inline-constant HAVE_EVP_CAMELLIA_256_CFB128 #t)
(define-inline-constant HAVE_DECL_EVP_CAMELLIA_256_CFB #t)
(define-inline-constant HAVE_EVP_CAMELLIA_256_OFB #t)
(define-inline-constant HAVE_EVP_SEED_ECB #t)
(define-inline-constant HAVE_EVP_SEED_CBC #t)
(define-inline-constant HAVE_EVP_SEED_CFB128 #t)
(define-inline-constant HAVE_DECL_EVP_SEED_CFB #t)
(define-inline-constant HAVE_EVP_SEED_OFB #t)
(define-inline-constant HAVE_EVP_CIPHER_TYPE #t)
(define-inline-constant HAVE_EVP_GET_CIPHERBYNAME #t)
(define-inline-constant HAVE_DECL_EVP_GET_CIPHERBYNID #t)
(define-inline-constant HAVE_DECL_EVP_GET_CIPHERBYOBJ #t)
(define-inline-constant HAVE_EVP_CIPHER_NID #t)
(define-inline-constant HAVE_DECL_EVP_CIPHER_NAME #t)
(define-inline-constant HAVE_EVP_CIPHER_BLOCK_SIZE #t)
(define-inline-constant HAVE_EVP_CIPHER_KEY_LENGTH #t)
(define-inline-constant HAVE_EVP_CIPHER_IV_LENGTH #t)
(define-inline-constant HAVE_EVP_CIPHER_FLAGS #t)
(define-inline-constant HAVE_DECL_EVP_CIPHER_MODE #t)
(define-inline-constant HAVE_EVP_CIPHER_CTX_INIT #t)
(define-inline-constant HAVE_EVP_CIPHER_CTX_CLEANUP #t)
(define-inline-constant HAVE_EVP_CIPHER_CTX_NEW #t)
(define-inline-constant HAVE_EVP_CIPHER_CTX_FREE #t)
(define-inline-constant HAVE_EVP_ENCRYPTINIT_EX #t)
(define-inline-constant HAVE_EVP_ENCRYPTFINAL_EX #t)
(define-inline-constant HAVE_EVP_ENCRYPTUPDATE #t)
(define-inline-constant HAVE_EVP_DECRYPTINIT_EX #t)
(define-inline-constant HAVE_EVP_DECRYPTUPDATE #t)
(define-inline-constant HAVE_EVP_DECRYPTFINAL_EX #t)
(define-inline-constant HAVE_EVP_CIPHERINIT_EX #t)
(define-inline-constant HAVE_EVP_CIPHERUPDATE #t)
(define-inline-constant HAVE_EVP_CIPHERFINAL_EX #t)
(define-inline-constant HAVE_EVP_CIPHER_CTX_SET_KEY_LENGTH #t)
(define-inline-constant HAVE_EVP_CIPHER_CTX_SET_PADDING #t)
(define-inline-constant HAVE_EVP_CIPHER_CTX_CTRL #t)
(define-inline-constant HAVE_EVP_CIPHER_CTX_CIPHER #t)
(define-inline-constant HAVE_EVP_CIPHER_CTX_NID #t)
(define-inline-constant HAVE_EVP_CIPHER_CTX_BLOCK_SIZE #t)
(define-inline-constant HAVE_EVP_CIPHER_CTX_KEY_LENGTH #t)
(define-inline-constant HAVE_EVP_CIPHER_CTX_IV_LENGTH #t)
(define-inline-constant HAVE_EVP_CIPHER_CTX_COPY #t)
(define-inline-constant HAVE_EVP_CIPHER_CTX_GET_APP_DATA #t)
(define-inline-constant HAVE_EVP_CIPHER_CTX_SET_APP_DATA #t)
(define-inline-constant HAVE_DECL_EVP_CIPHER_CTX_TYPE #t)
(define-inline-constant HAVE_EVP_CIPHER_CTX_FLAGS #t)
(define-inline-constant HAVE_DECL_EVP_CIPHER_CTX_MODE #t)
(define-inline-constant HAVE_EVP_CIPHER_CTX_RAND_KEY #t)
(define-inline-constant HAVE_EVP_CIPHER_PARAM_TO_ASN1 #t)
(define-inline-constant HAVE_EVP_CIPHER_ASN1_TO_PARAM #t)
(define-inline-constant HAVE_EVP_CIPHER_CTX_SET_FLAGS #t)
(define-inline-constant HAVE_EVP_CIPHER_CTX_CLEAR_FLAGS #t)
(define-inline-constant HAVE_EVP_CIPHER_CTX_TEST_FLAGS #t)
(define-inline-constant HAVE_EVP_CIPHER #t)
(define-inline-constant HAVE_BUF_MEM_NEW #t)
(define-inline-constant HAVE_BUF_MEM_FREE #t)
(define-inline-constant HAVE_BUF_MEM_GROW #t)
(define-inline-constant HAVE_BUF_MEM_GROW_CLEAN #t)
(define-inline-constant HAVE_BUF_STRDUP #t)
(define-inline-constant HAVE_BUF_STRNDUP #t)
(define-inline-constant HAVE_BUF_MEMDUP #t)
(define-inline-constant HAVE_BUF_REVERSE #t)
(define-inline-constant HAVE_BUF_STRLCPY #t)
(define-inline-constant HAVE_BUF_STRLCAT #t)
(define-inline-constant HAVE_ERR_LOAD_BUF_STRINGS #t)

;;;; done

)

;;; end of file
